"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var MongoModule_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.MongoModule = void 0;
const common_1 = require("@nestjs/common");
const constants_1 = require("../lib/constants");
const utils_1 = require("../lib/utils");
const mongodb_1 = require("../lib/mongodb");
const metadata_1 = require("../lib/metadata");
const collection_1 = require("../lib/collection");
const mongo_service_1 = require("./mongo.service");
let MongoModule = MongoModule_1 = class MongoModule {
    static forRoot(options) {
        const providers = this.getProviders();
        return {
            global: true,
            module: MongoModule_1,
            exports: providers.map(({ provide }) => provide),
            providers: [
                {
                    provide: constants_1.MONGO_MODULE_OPTIONS,
                    useValue: options
                },
                ...providers
            ]
        };
    }
    static forRootAsync({ inject, useFactory }) {
        const providers = this.getProviders();
        return {
            global: true,
            module: MongoModule_1,
            exports: providers.map(({ provide }) => provide),
            providers: [
                {
                    inject,
                    useFactory,
                    provide: constants_1.MONGO_MODULE_OPTIONS
                },
                ...providers
            ]
        };
    }
    static forFeature(entities) {
        const providers = entities.map(entity => {
            const collectionName = Reflect.getMetadata(constants_1.ENTITY_COLLECTION_NAME, entity);
            if (!collectionName) {
                throw new mongodb_1.MongoRuntimeError(`Entity "${entity.name}" is not registered with decorator @Entity(name: string)`);
            }
            return {
                provide: (0, utils_1.getCollectionToken)(entity),
                inject: [
                    {
                        optional: true,
                        token: mongodb_1.MongoDatabase
                    }
                ],
                useFactory: (db) => {
                    if (!db) {
                        throw new mongodb_1.MongoRuntimeError('Failed to find MongoDatabase. Is MongoModule registered properly?');
                    }
                    metadata_1.CollectionMetadata.collect(entity);
                    return new collection_1.Collection(db, collectionName);
                }
            };
        });
        return {
            module: MongoModule_1,
            providers,
            exports: providers.map(({ provide }) => provide)
        };
    }
    static getProviders() {
        return [
            {
                provide: mongodb_1.MongoClient,
                inject: [constants_1.MONGO_MODULE_OPTIONS],
                useFactory: (options) => new mongodb_1.MongoClient(options.uri, options.client)
            },
            {
                provide: mongodb_1.MongoDatabase,
                inject: [constants_1.MONGO_MODULE_OPTIONS, mongodb_1.MongoClient],
                useFactory: (options, client) => client.db(undefined, options.database)
            },
            {
                provide: mongo_service_1.MongoService,
                useClass: mongo_service_1.MongoService
            }
        ];
    }
};
exports.MongoModule = MongoModule;
exports.MongoModule = MongoModule = MongoModule_1 = __decorate([
    (0, common_1.Module)({})
], MongoModule);
//# sourceMappingURL=mongo.module.js.map