"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var MongoService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.MongoService = void 0;
const common_1 = require("@nestjs/common");
const mongodb_1 = require("../lib/mongodb");
const constants_1 = require("../lib/constants");
const decorators_1 = require("../lib/decorators");
const metadata_1 = require("../lib/metadata");
const collection_1 = require("../lib/collection");
let MongoService = MongoService_1 = class MongoService {
    db;
    client;
    logger = new common_1.Logger(MongoService_1.name);
    constructor(db, client) {
        this.db = db;
        this.client = client;
    }
    async onApplicationBootstrap() {
        this.logger.log('Setting up collections');
        const collections = await this.db.listCollections().toArray();
        for (const [entity, metadata] of metadata_1.CollectionMetadata.registry) {
            const collectionName = Reflect.getMetadata(constants_1.ENTITY_COLLECTION_NAME, entity);
            if (!collectionName) {
                continue;
            }
            const collection = new collection_1.Collection(this.db, collectionName);
            const isCollectionCreated = collections.some(({ name }) => name === collectionName);
            if (!isCollectionCreated) {
                await this.db.createCollection(collectionName).catch(error => {
                    this.logger.debug(`Failed to create collection: ${collectionName}, reason:`, error.stack);
                });
            }
            await metadata.synchronize(collection);
        }
    }
    async onApplicationShutdown() {
        this.logger.log('Shutting down');
        await this.client.close();
    }
};
exports.MongoService = MongoService;
exports.MongoService = MongoService = MongoService_1 = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, decorators_1.InjectDatabase)()),
    __param(1, (0, decorators_1.InjectClient)()),
    __metadata("design:paramtypes", [mongodb_1.MongoDatabase,
        mongodb_1.MongoClient])
], MongoService);
//# sourceMappingURL=mongo.service.js.map